@@ -782,6 +782,8 @@ void *hunt(void *arg)
 			    case ETH_P_IP:
 				ALIGNPOINTERS_IP(ethh, iph);
 				p->p_iph = iph;
+			        if (in_cksum((unsigned short *) iph,
+					     IP_HDR_LENGTH(iph)) == 0) {
 				if (mac_learn_from_ip)
 					mac_ip_learn(p);
 				process_ip(p);
@@ -791,23 +793,31 @@ void *hunt(void *arg)
 					ALIGNPOINTERS_TCP(iph, p->p_hdr.p_tcph,
 							  p->p_data);
 					p->p_data_len = TCP_DATA_LENGTH(iph, p->p_hdr.p_tcph);
-					conn_update_table(p, ethh, iph);
-					fast_tcp_process(p);
-					process_tcp(p);
+					if (ip_in_cksum(iph, (unsigned short *) p->p_hdr.p_tcph, 
+					    IP_DATA_LENGTH(iph)) == 0) {
+						conn_update_table(p, ethh, iph);
+						fast_tcp_process(p);
+						process_tcp(p);
+					}
 					break;
 				    case IPPROTO_UDP:
 					p->p_type = PACKET_UDP;
 					ALIGNPOINTERS_UDP(iph, p->p_hdr.p_udph, 
 							  p->p_data);
+					/* check the UDP checksum */
 					process_udp(p);
 					break;
 				    case IPPROTO_ICMP:
 					p->p_type = PACKET_ICMP;
 					ALIGNPOINTERS_ICMP(iph, p->p_hdr.p_icmph,
 							   p->p_data);
-					process_icmp(p);
+					if (in_cksum((unsigned short *) p->p_hdr.p_icmph,
+					    IP_DATA_LENGTH(iph)) == 0) {
+						process_icmp(p);
+					}
 					break;
 				}
+				}
 				break;
 			    case ETH_P_ARP:
 				p->p_type = PACKET_ARP;
